/***********************************************************************
 ipcard.c - Pocket Reference for the Internet Protocol header
 v1.01

 Copyright (c) 2003 Dirk Loss <dloss@uni-muenster.de>

 All rights reserved.

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, and/or sell copies of thei
 Software, and to permit persons to whom the Software is furnished to do so,
 provided that the above copyright notice(s) and this permission notice appear
 in all copies of the Software and that both the above copyright notice(s)
 and this permission notice appear in supporting documentation.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE BE
 LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR 
 ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

 Except as contained in this notice, the name of a copyright holder shall
 not be used in advertising or otherwise to promote the sale, use or other
 dealings in this Software without prior written authorization of the
 copyright holder.  

***************************************************************************
 IP-Card is heavily based on the sample "Hello, world!" program for Palm OS
 Revision 1.1 by Warren Young which was put in the public domain.

 Build with PRC-Tools 2.0 and Palm OS SDK 3.5 or better.
***********************************************************************/

#include <PalmOS.h>

#include "resource.h"


////////////////////////////////////////////////////////////////////////
// Globals

// Pointer to the currently-active form
static FormPtr gpForm;


//// StartApplication //////////////////////////////////////////////////
// Called when application is starting

static Err StartApplication() 
{
	FrmGotoForm(IpcardForm);
	return 0;
}


//// StopApplication ///////////////////////////////////////////////////
// Called when application is being shut down.

static void StopApplication() 
{
	FrmCloseAllForms();
}


//// IpcardFormEventHandler /////////////////////////////////////////////
// Handle events for the IP-Card form.

Boolean IpcardFormEventHandler(EventPtr event) 
{
	Boolean handled = false;

	switch (event->eType) {
		case frmOpenEvent: {
			gpForm = FrmGetActiveForm();
			FrmDrawForm(gpForm);
			handled = true;
			break;
		}

		case frmCloseEvent:
			FrmEraseForm(gpForm);
			FrmDeleteForm(gpForm);
			gpForm = 0;
			handled = true;
			break;

		default:
			// -Wall warning eater: switch must handle all enum
			// elements.
			break;
	}

	return handled;
}


//// ApplicationEventHandler /////////////////////////////////////////////
// Handle global events, and pass form-specific events to each form's
// event handler.

static Boolean ApplicationEventHandler(EventPtr event) 
{
	Boolean handled = false;

	switch (event->eType) {
		case frmLoadEvent: {
			FormPtr pForm = FrmInitForm(event->data.frmLoad.formID);
			FrmSetActiveForm(pForm);
			FrmSetEventHandler(pForm, IpcardFormEventHandler);
			handled = true;
			break;
		}
			
		case menuEvent:
			switch (event->data.menu.itemID) {
				case HelpMenuAbout:
					FrmAlert(AboutAlert);
					break;
			}

			handled = true;
			break;

		default:
			// -Wall warning eater: switch must handle all enum
			// elements.
			break;
	}

	return handled;
}


//// EventLoop /////////////////////////////////////////////////////////
// Global event loop.

static void EventLoop() 
{
	EventType event;
	UInt16 error;
	do {
		EvtGetEvent(&event, evtWaitForever);
		
		if (!SysHandleEvent(&event)) {
			if (!MenuHandleEvent(0, &event, &error)) {
				if (!ApplicationEventHandler(&event)) {
					FrmDispatchEvent(&event);
				}
			}
		}
	}
	while (event.eType != appStopEvent);
}


//// PilotMain /////////////////////////////////////////////////////////
// It all starts here...

UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags) 
{
	Err err = 0;

	if (launchCode == sysAppLaunchCmdNormalLaunch) {
		if ((err = StartApplication()) == 0) {
			EventLoop();
			StopApplication();
		}
	}

	return err;
}

